@@ -1,6 +1,6 @@
 /*
  * QuickJS Read Eval Print Loop
- * 
+ *
  * Copyright (c) 2017-2020 Fabrice Bellard
  * Copyright (c) 2017-2020 Charlie Gordon
  *
@@ -22,16 +22,16 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  * THE SOFTWARE.
  */
-import * as std from "std";
-import * as os from "os";
-import * as bjson from "bjson";
+import * as std from "qjs:std";
+import * as os from "qjs:os";
+import * as bjson from "qjs:bjson";
 
 (function(g) {
     /* add 'bjson', 'os' and 'std' bindings */
     g.bjson = bjson;
     g.os = os;
     g.std = std;
-    
+
     /* close global objects */
     var Object = g.Object;
     var String = g.String;
@@ -149,12 +149,12 @@ import * as bjson from "bjson";
     var term_read_buf;
     var term_width;
     /* current X position of the cursor in the terminal */
-    var term_cursor_x = 0; 
-    
+    var term_cursor_x = 0;
+
     function termInit() {
         var tab;
         term_fd = std.in.fileno();
-        
+
         /* get the terminal size */
         term_width = 80;
         if (os.isatty(term_fd)) {
@@ -181,14 +181,14 @@ import * as bjson from "bjson";
         /* send Ctrl-C to readline */
         handle_byte(3);
     }
-    
+
     function term_read_handler() {
         var l, i;
         l = os.read(term_fd, term_read_buf.buffer, 0, term_read_buf.length);
         for(i = 0; i < l; i++)
             handle_byte(term_read_buf[i]);
     }
-    
+
     function handle_byte(c) {
         if (!utf8) {
             handle_char(c);
@@ -206,12 +206,12 @@ import * as bjson from "bjson";
             handle_char(c);
         }
     }
-    
+
     function is_alpha(c) {
         return typeof c === "string" &&
             ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z'));
     }
-    
+
     function is_digit(c) {
         return typeof c === "string" && (c >= '0' && c <= '9');
     }
@@ -243,7 +243,7 @@ import * as bjson from "bjson";
         d = c.codePointAt(0); /* can be NaN if empty string */
         return d >= 0xdc00 && d < 0xe000;
     }
-    
+
     function is_balanced(a, b) {
         switch (a + b) {
         case "()":
@@ -282,7 +282,7 @@ import * as bjson from "bjson";
                 } else {
                     l = Math.min(term_width - 1 - term_cursor_x, delta);
                     print_csi(l, "C"); /* right */
-                    delta -= l; 
+                    delta -= l;
                     term_cursor_x += l;
                 }
             }
@@ -410,7 +410,12 @@ import * as bjson from "bjson";
 
     function backward_word() {
         cursor_pos = skip_word_backward(cursor_pos);
-    }        
+    }
+
+    function clear_screen() {
+        directives["clear"]();
+        return -2;
+    }
 
     function accept_line() {
         std.puts("\n");
@@ -591,7 +596,7 @@ import * as bjson from "bjson";
             readline_print_prompt();
         }
     }
-    
+
     function reset() {
         cmd = "";
         cursor_pos = 0;
@@ -757,7 +762,7 @@ import * as bjson from "bjson";
             readline_print_prompt();
         }
     }
-    
+
     var commands = {        /* command table */
         "\x01":     beginning_of_line,      /* ^A - bol */
         "\x02":     backward_char,          /* ^B - backward-char */
@@ -770,6 +775,7 @@ import * as bjson from "bjson";
         "\x09":     completion,             /* ^I - history-search-backward */
         "\x0a":     accept_line,            /* ^J - newline */
         "\x0b":     kill_line,              /* ^K - delete to end of line */
+        "\x0c":     clear_screen,           /* ^L - clear screen */
         "\x0d":     accept_line,            /* ^M - enter */
         "\x0e":     next_history,           /* ^N - down */
         "\x10":     previous_history,       /* ^P - up */
@@ -777,6 +783,7 @@ import * as bjson from "bjson";
         "\x12":     alert,                  /* ^R - reverse-search */
         "\x13":     alert,                  /* ^S - search */
         "\x14":     transpose_chars,        /* ^T - transpose */
+        "\x17":     backward_kill_word,     /* ^W - backward_kill_word */
         "\x18":     reset,                  /* ^X - cancel */
         "\x19":     yank,                   /* ^Y - yank */
         "\x1bOA":   previous_history,       /* ^[OA - up */
@@ -833,9 +840,9 @@ import * as bjson from "bjson";
         cursor_pos = cmd.length;
         history_index = history.length;
         readline_cb = cb;
-        
+
         prompt = pstate;
-    
+
         if (mexpr) {
             prompt += dupstr(" ", plen - prompt.length);
             prompt += ps2;
@@ -921,7 +928,7 @@ import * as bjson from "bjson";
         } else {
             alert(); /* beep! */
         }
-        
+
         cursor_pos = (cursor_pos < 0) ? 0 :
             (cursor_pos > cmd.length) ? cmd.length : cursor_pos;
         update();
@@ -1516,7 +1523,7 @@ import * as bjson from "bjson";
     function cmd_readline_start() {
         readline_start(dupstr("    ", level), readline_handle_cmd);
     }
-    
+
     function readline_handle_cmd(expr) {
         if (!handle_cmd(expr)) {
             cmd_readline_start();
@@ -1541,30 +1548,21 @@ import * as bjson from "bjson";
             return false;
         }
         mexpr = "";
-        
-        eval_and_print_start(expr, true);
+
+        eval_and_print(expr);
 
         return true;
     }
 
-    function eval_and_print_start(expr, is_async) {
+    function eval_and_print(expr) {
         var result;
-        
-        try {
-            if (use_strict)
-                expr = '"use strict"; void 0;' + expr;
-            eval_start_time = os.now();
-            /* eval as a script */
-            result = std.evalScript(expr, { backtrace_barrier: true, async: is_async });
-            if (is_async) {
-                /* result is a promise */
-                result.then(print_eval_result, print_eval_error);
-            } else {
-                print_eval_result({ value: result });
-            }
-        } catch (error) {
-            print_eval_error(error);
-        }
+
+        if (use_strict)
+            expr = '"use strict"; void 0;' + expr;
+        eval_start_time = os.now();
+        /* eval as a script */
+        result = Promise.try(std.evalScript, expr, { backtrace_barrier: true, async: true });
+        result.then(print_eval_result, print_eval_error);
     }
 
     function print_eval_result(result) {
@@ -1578,24 +1576,31 @@ import * as bjson from "bjson";
     }
 
     function print_eval_error(error) {
-        std.puts(colors[styles.error]);
+        if (show_colors) {
+            std.puts(colors[styles.error]);
+        }
         if (error instanceof Error) {
-            console.log(error);
+            std.puts(error);
+            std.puts('\n');
             if (error.stack) {
                 std.puts(error.stack);
             }
         } else {
             std.puts("Throw: ");
-            console.log(error);
+            std.puts(error);
+            std.puts('\n');
+        }
+
+        if (show_colors) {
+            std.puts(colors.none);
         }
-        std.puts(colors.none);
 
         handle_cmd_end();
     }
 
     function handle_cmd_end() {
         level = 0;
-        
+
         /* run the garbage collector after each command */
         std.gc();
 
@@ -1872,6 +1877,10 @@ import * as bjson from "bjson";
                 styles = themes.light;
             }
         }
+        s = std.getenv("NO_COLOR"); // https://no-color.org/
+        if (s && +s[0] !== 0) {
+            show_colors = false;
+        }
     }
 
     load_config();
